
#define LED_PORT PORTB
#define LED_DDR DDRB
#define LED_PIN PINB
#define LED_ACCEPT PB6
#define LED_DENIED PB7
#define BTN_DDR DDRD
#define BTN_PORT PORTD
#define BTN_PIN PD4

#include "constants.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>

#include "em4100.h"
#include "lcd.h"
#include "rs232.h"
#include "timer1.h"


char g_Header[16];

void delay(uint16_t ms)
{
	for(; ms>0; ms--)
	_delay_ms(1);
}

inline void led_toggle(uint8_t pin)
{
	if (LED_PIN & (1<<pin))
		LED_PORT &= ~(1<<pin);
	else
		LED_PORT |= ( 1 << pin );
}


inline uint8_t tagToString(char* buffer, uint8_t* tag)
{
	uint8_t index = 0;
	
	for (uint8_t j=0; j<5; j++)
	{
		uint8_t nibble = (tag[j] >> 4) & 0x0f;
		if (nibble > 9)
		nibble += 'A'-10;
		else
		nibble += '0';
		buffer[index++] = nibble;
		
		nibble = tag[j] & 0x0f;
		if (nibble > 9)
		nibble += 'A'-10;
		else
		nibble += '0';
		buffer[index++] = nibble;
	}
	
	return index;
}

inline void sendCommand(char id, char* data)
{
	UART_putchar(id);
	UART_puts(data);
}
void renderHeader(void)
{
	lcd_clear();
	lcd_stringCenter(g_Header, 1);
	lcd_stringCenter_P(PSTR("Bereit"), 2);
}

int main(void)
{
	bool btnReleased = true;
	timer1_init();
	lcd_init();
	UART_init();
	rfid_init();
	lcd_clear();
	
	LED_DDR |= (1<<DDB6) | (1<<DDB7);
	BTN_DDR &= ~(1<<BTN_PIN); // eingang
	
	TCCR0B = 1<<CS01 | 1<<CS00; //Timer/Counter0 prescaler 64
	MCUCR |= 1<<ISC10; //Any logical change on INT1 generates an interrupt request
	GIMSK |= 1<<INT1; //External interrupt request 1 enable (demod_out from RFID chip)
	
	sei();
	
	sendCommand(CMD_HEADER, "");
	uint8_t tag[5];
	char tagString[11];
	strcpy(g_Header,"~Zugang~");
	renderHeader();
	rfid_enable();
	while ( true )
	{
		if ( em4095_read_tag(tag) )
	    {
			rfid_disable();
			tagToString(tagString, tag);
			
			lcd_clear();
			lcd_stringAt_P(PSTR("ID:"), 0, 1);
			lcd_stringAt(tagString, 4, 1);
			lcd_stringCenter_P(PSTR("ID Check..."), 2);
			sendCommand(CMD_TAG, tagString);
			timer1_enable();
		}
		
		if(timer1_endwaiting)
		{
			lcd_clear();
			lcd_stringCenter_P(PSTR("No answer!"),1 );
			delay(1000);
			timer1_endwaiting = false;
			rfid_enable();
			renderHeader();
		}
		
		
		if ( (PIND & (1<<BTN_PIN)) )
		{
			if ( btnReleased )
			{
				btnReleased = false;
				
				sendCommand(CMD_PARTY, "");
			}
		}
		else
		{
			btnReleased = true;
		}
		
		if(UART_receive_complete())
		{
			timer1_disable();
			lcd_clear();
			char* uart_string = UART_received_string();
			char cmd = uart_string[0];
			if ( cmd == CMD_HEADER ) 
			{
				strcpy(g_Header, uart_string+1);
				renderHeader();
			}
			
			else if ( cmd == CMD_TAG )
			{
				char accepted = uart_string[1];
				
				if ( accepted == UART_TRUE )
				{
					lcd_stringCenter_P(PSTR("Welcome"), 1);
					char* name = ( uart_string + 2);
					lcd_stringCenter(name, 2);
					led_toggle(LED_ACCEPT);
					delay(2500);
				}
				else
				{
					lcd_stringCenter_P(PSTR("Access denied"), 1);
					led_toggle(LED_DENIED);
					delay(2000);
				}
				renderHeader();
				rfid_enable();
			}
			else if ( cmd == CMD_CONFIG)
			{
				bool inConfig = ( uart_string[1] == '1' ); // whether we should 'lock'
				
				if ( inConfig )
				{
					rfid_disable();
					lcd_clear();
					lcd_stringCenter_P(PSTR("Config..."), 1);
				}
				else
				{
					rfid_enable();
					renderHeader();
				}
			}
			else
			{
				lcd_stringCenter_P(PSTR("Unknown command"), 1);
				lcd_stringCenter(uart_string, 2);
				led_toggle(LED_ACCEPT);
				led_toggle(LED_DENIED);
				delay(1000);
				rfid_enable();
				renderHeader();
			}

			UART_receive_completed();
			LED_PORT &= ~(1<<LED_ACCEPT);
			LED_PORT &= ~(1<<LED_DENIED);
		}
	}
	
	return 0;
}
